function getMuscPath(muscName, originName, insertName, noSlices)
% getMuscPath estimaes line of action for muscle 'muscName' (.obj files). 
% Requires estimaate of anatomical origin ('originName') insertion
% ('insertName', both .mat files generated by getMuscAttach.m). Line of
% action will be estimated as centre of noSlices number of axial
% cross-sections taken on vector from origin to insertion. 

% load stuff
[mV,mF] = loadWobj(muscName);
load(originName);
origin = attach;
load(insertName);
insert = attach;

%% transform so that origin is [0 0 0] and origin->insert = world X;
cent = origin;
muscVec = insert-origin;
muscCS = getCS(muscVec);

mVTrans = (mV - cent) * muscCS';


%% get slices

muscSlices = sliceMesh(mVTrans,mF,noSlices,1);

%% get centroids of slices

% get slices
allSliceLoops = fieldnames(muscSlices);
% ignore ones not containing '_In'
allSliceLoops(cellfun(@isempty,regexp(allSliceLoops,'_In'))) = [];
sliceCentres = zeros(noSlices,3);
% set array size
arrayDim = 50;

for loop1 = 1:noSlices
    
    currSliceLoops = allSliceLoops(strmatch(['Slice_',num2str(loop1),'_'], allSliceLoops));
    currSliceCentres = zeros(length(currSliceLoops),3);
    currSliceMasses = zeros(length(currSliceLoops),1);
    
    % get bounding box of ALL sliceloops
    loopCoordsAll = [0 0 0];
    for loop2 =1:length(currSliceLoops)
        loopFieldName = cell2mat(currSliceLoops(loop2));
        loopCoords = muscSlices.(loopFieldName);
        loopX = loopCoords(1,1);
        loopCoordsAll = [loopCoordsAll; loopCoords];
    end
    loopCoordsAll(1,:) = [];
    loopCoordsAll = loopCoordsAll(:,2:3);
    % get bounding box and make arrayDimXarrayDim array of points within it
    sliceBBox = [min(loopCoordsAll); max(loopCoordsAll)];
    pntArrayY = makeRange(sliceBBox(1,1),sliceBBox(2,1),arrayDim);
    pntArrayZ = makeRange(sliceBBox(1,2),sliceBBox(2,2),arrayDim);
    pntArrayX = repmat(loopX,arrayDim*arrayDim,1);
    pntArray = zeros(arrayDim*arrayDim,2);    
        
    for loop2 =1:length(currSliceLoops)
        
        loopFieldName = cell2mat(currSliceLoops(loop2));
        
        % get coords and collapse to YZ plane
        loopCoords = muscSlices.(loopFieldName);
        loopCoords = loopCoords(:,2:3);
        
        readCount = 1;
        for loop3 = 1:arrayDim:arrayDim*arrayDim
            pntArray(loop3:loop3+arrayDim-1,:) = ...
                [repmat(pntArrayY(readCount),arrayDim,1) pntArrayZ'];
            readCount = readCount+1;
        end
        inPnts = inpoly(pntArray, loopCoords);
        pntArrayIn = pntArray(inPnts,:);
        pntArrayInX = pntArrayX(inPnts);
        pntArrayIn3D = [pntArrayInX pntArrayIn];
        
        % assuming all cuboids of unit mass, get mass as no of points and
        % centre as mean point
        currSliceCentres(loop2,:) = mean(pntArrayIn3D,1);
        currSliceMasses(loop2) = size(pntArrayIn3D,1);

    end
    
    % cull NaNs, think it's just a miniscule poly patch problem...
    currSliceCentres(isnan(currSliceCentres(:,1)),:) = [];
    currSliceMasses(currSliceMasses==0) = [];   
    
    % get system centre (CoM, kinda)
    if size(currSliceMasses,1)>1
       % sum mass
        sliceMass = sum(currSliceMasses);

        % calculate CoM
        sliceCentres(loop1,:) = (1/sliceMass) * sum(currSliceCentres .* ...
            [currSliceMasses currSliceMasses currSliceMasses]);
    else
        sliceCentres(loop1,:) = currSliceCentres;
    end
    
end

% output
path = (sliceCentres*muscCS)+cent;


%% draw
figure(1)
clf
subplot(1,2,1)
hold on
trimesh(mF,mV(:,1),mV(:,2),mV(:,3),'edgecolor','k', 'facecolor',[0.6 0.6 0.6]);
%plot3(path(:,1),path(:,2),path(:,3),'go','markersize',15,'markerfacecolor','green');
axis equal
view(3)

subplot(1,2,2)
hold on

for loop1 = 1:noSlices
    
    SliceLoops = allSliceLoops(strmatch(['Slice_',num2str(loop1),'_'], allSliceLoops));
    
    % DRAW SLICE
    
    for i=1:length(SliceLoops)
        
        loopFieldName = cell2mat(SliceLoops(i));
        
        loopCoords = (muscSlices.(loopFieldName)*muscCS)+cent;
        
        if regexp(loopFieldName,'_In')>0
            plot3(loopCoords(:,1),loopCoords(:,2),loopCoords(:,3),'r')
        end
        
    end
end
plot3(path(:,1),path(:,2),path(:,3),'go','markersize',15,'markerfacecolor','green');
axis equal
view(3)

%% output

% make name
outName = [regexprep(muscName,'.obj',''),'_path'];

% output
save([outName,'.mat'], 'path');
csvwrite([outName,'.csv'],path);
saveas(1,[outName,'.fig']);

end

%% embedded functions

function [muscSlices] = sliceMesh(v,f,noSlices,slicePlane);
% Author: Vivian Allen (mrvivianallen@gmail.com)

% Generates a struct array of XYZ coordinates for sliceloops through a 
% convex mesh composed of tri and quad (or tri or quad only) polys, with 
% whether each slice loop encloses an internal or external portion of the 
% mesh. 

% REQUIRED FUNCTIONS (must be in same folder):
% FindEdgeIntersectsIndex.m
% inpoly.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% INPUTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% CALCULATE/INITIALISE

% remove old storage struct if present
clear muscSlices

% check for compound mesh(Quads AND Tris), set indicator. Seperate out 
% tri and quad polys if compound mesh found.
if sum(sum(f==0))>0;
    compoundMesh = 1;
    [rows,~] = find(f==0);
    tris = f(rows,:);
    tris(:,4) = [];
    quads = f;
    quads(rows,:) = [];
else
    compoundMesh = 0;
end

%non slice planes = the other ones!
nonSlicePlanes = [1 2 3];
nonSlicePlanes(slicePlane) = [];


% get mesh dimensions in slice plane.
meshSliceAxismax = max(v(:,slicePlane));
meshSliceAxismin = min(v(:,slicePlane));

% use body dimensions and noSlices to determine slice thickness
sliceThickness = (meshSliceAxismax-meshSliceAxismin)/noSlices;

% generate list of sliceHorizons, starting from 1/2 slice thickness up from
% object min to 1/2 slice thickness down from object max (in slice axis)
sliceHorizons = (meshSliceAxismin + (sliceThickness/2)):sliceThickness: ... 
    (meshSliceAxismax-(sliceThickness/2));

%% list components of polys in the slice plane axis 
if compoundMesh==0;
    polySliceAxisCoords = zeros(size(f));
    for i=1:size(f,2)
        polySliceAxisCoords(:,i) = v(f(:,i),slicePlane);
    end
else
    polySliceAxisCoordsTris = zeros(size(tris));
    polySliceAxisCoordsQuads = zeros(size(quads));
    for i = 1:size(tris,2)
        polySliceAxisCoordsTris(:,i) = v(tris(:,i),slicePlane);
    end
    for i = 1:size(quads,2)
        polySliceAxisCoordsQuads(:,i) = v(quads(:,i),slicePlane);
    end
end

%% PRIMARY LOOP
for i=1:size(sliceHorizons,2)
    
    % find intersection coordinates for current slice with mesh polys (see
    % integrated function at the bottom). Run once if homogenous mesh (tris
    % or quads only)..
    if compoundMesh ==0;
        intersectCoords = FindEdgeIntersectsIndex(f, v, sliceHorizons(i), ...
            polySliceAxisCoords,slicePlane);
    else % ...or run for each poly type if compound mesh.
        intersectCoordsTris = FindEdgeIntersectsIndex(tris, v, sliceHorizons(i), ...
        polySliceAxisCoordsTris, slicePlane);
        intersectCoordsQuads = FindEdgeIntersectsIndex(quads, v, sliceHorizons(i), ...
        polySliceAxisCoordsQuads, slicePlane);
       
        % combine quad and tri output, ignore either if they are empty (no
        % intercepts with current slice in either quad or tri part of mesh)
        if sum(sum(intersectCoordsTris))>0 && sum(sum(intersectCoordsQuads))>0 
        intersectCoords = [intersectCoordsTris; intersectCoordsQuads];
        elseif sum(sum(intersectCoordsTris))==0
        intersectCoords = intersectCoordsQuads;
        else
        intersectCoords = intersectCoordsTris;
        end
        % re-write durable ID (pair number) for each pair of intersection
        % points to avoid dupes (as the two outputs have been joined).
        intersectCoords(:,1) = 1:size(intersectCoords,1);
        intersectCoords(:,5) = 1:size(intersectCoords,1);    
    
    end
    
    
    % order intersect coords into a loop, or a nested series of loops!
    % first turn intersect coordinates from a list of pairs into a vertical
    % list (helps sorting). Pair number stops pairs being disassociated.
    unsortedCoords = [intersectCoords(:,1:4); intersectCoords(:,5:8)];
    % declare storage matrix for coords sorted into pairs, to be populated
    % by loop below.
    loopCoords = zeros(size(unsortedCoords));
    % START SEARCH: Start with first intersect point, so make the first two
    % entries in the  loop coordinates the first pair of intersect
    % coordinates.
    [rows ~] = find(unsortedCoords(:,1)==1);
    loopCoords(1:2,:) = unsortedCoords(rows,:);
    % remove this pair from the unsorted Coordinates.
    unsortedCoords(rows,:) = [];
    % set dynamic index for loop coordinates
    loopCoordsIndexList = 3:2:size(loopCoords,1);
    currentLoopCoordIndex = 1;
    %dynamic index for loop number
    loopNoIndex = 1; 
   
    % loop through unsorted coordinates and sort them into pairs that share
    % one coordinate with the preceeding and one coordinate with the
    % proceeding pairs (loop segments)
    while size(unsortedCoords,1)>0
        
        % find twins of current loop coord
        twinInd = strmatch(loopCoords(loopCoordsIndexList( ...
            currentLoopCoordIndex)-1,2:4),unsortedCoords(:,2:4), ...
            'exact');
        if isempty(twinInd)==1;
            % end of loop has been reached! 
            % strip zeros from end (if present)
            [rows ~] = find(loopCoords(:,1)==0);
            loopCoords(rows,:) = [];
            % strip durable pair indices
            loopCoords(:,1) = [];
            % remove duplicates: first find sort index for unique rows
            [~, sortIndex] = unique(loopCoords,'rows');
            % then sort index into numerical order (same sense as original 
            % array)
            sortIndex = sort(sortIndex);
            % then trim out duplicates in coords using sortIndex
            loopCoords =loopCoords(sortIndex,:);
            % make last entry = first entry (loop)
            loopCoords = [loopCoords; loopCoords(1,:)];
            
            % use loopNoIndex to output loopCoords to dynically named
            % field in the muscSlices struct
            eval(['FieldName = ''Slice_',num2str(i),'_',num2str(loopNoIndex),''';']);
            muscSlices.(FieldName) = loopCoords;
            % move loopNoIndex along one notch
            loopNoIndex = loopNoIndex+1;
            
            % reinitialise search:
            % remake loopCoords with first two of remaining unsortedCoords
            % as first pair
            loopCoords = zeros(size(unsortedCoords));
            [rows ~] = find(unsortedCoords(:,1)==unsortedCoords(1,1));
            loopCoords(1:2,:) = unsortedCoords(rows,:);
            unsortedCoords(rows,:) = [];
            % reset loopCoordIndex
            currentLoopCoordIndex = 1;
            
            %look again for twin            
            twinInd = strmatch(loopCoords(loopCoordsIndexList( ...
            currentLoopCoordIndex)-1,2:4),unsortedCoords(:,2:4), ...
            'exact');
        end
        
        if size(twinInd,1)==1
        % pair up twins...
        loopCoords(loopCoordsIndexList(currentLoopCoordIndex),:) = ...
            unsortedCoords(twinInd,:);
        unsortedCoords(twinInd,:) = [];
        
        % look for twins spouse
        twinSpouseInd = find(unsortedCoords(:,1)==loopCoords....
            (loopCoordsIndexList(currentLoopCoordIndex),1));
        loopCoords(loopCoordsIndexList(currentLoopCoordIndex)+1,:) ...
            = unsortedCoords(twinSpouseInd,:);
        unsortedCoords(twinSpouseInd,:) = [];
        end
        
        currentLoopCoordIndex = currentLoopCoordIndex+1;
    end
   
    % write last loop to array
    [rows ~] = find(loopCoords(:,1)==0);
    loopCoords(rows,:) = [];
    % strip durable pair indices
    loopCoords(:,1) = [];
    % remove duplicates: first find sort index for unique rows
    [~, sortIndex] = unique(loopCoords,'rows');
    % then sort index into numerical order (same sense as original 
    % array)
    sortIndex = sort(sortIndex);
    % then trim out duplicates in coords using sortIndex
    loopCoords =loopCoords(sortIndex,:);
    % make last entry = first entry (loop)
    loopCoords = [loopCoords; loopCoords(1,:)];
       
   % use loopNoIndex to output loopCoords to dynically named
   % field in the muscSlices struct
   eval(['FieldName = ''Slice_',num2str(i),'_',num2str(loopNoIndex),''';']);
   muscSlices.(FieldName) = loopCoords;
    
   % nest loops! Use the inpoly function (taken from matlab central, 
   % http://www.mathworks.com/matlabcentral/fileexchange/10391)
   
   % create storage array for loop nesting indices
   loopNests = zeros(loopNoIndex,loopNoIndex);
    
    for p = 1:loopNoIndex
        % load criteria loop
         eval(['currentLoop = muscSlices.Slice_',num2str(i),'_',num2str(p),';']);
         otherLoops = 1:loopNoIndex;
         otherLoops(p)=[];
         for q = 1:length(otherLoops)
            % load test loops
            eval(['testLoop = muscSlices.Slice_',num2str(i),'_',num2str(otherLoops(q)),';']);
            loopNests(p,otherLoops(q)) = sum(inpoly([testLoop(:,nonSlicePlanes(1))...
                    , testLoop(:,nonSlicePlanes(2))], ...
                    [currentLoop(:,nonSlicePlanes(1)), ...
                    currentLoop(:,nonSlicePlanes(2))])) == size(testLoop,1);
         end      
    end
     
    %define loops as inside/outside - append struct fieldname with 'in' or
    %'out' to indicate
    for p = 1:loopNoIndex
         eval(['currentLoop = muscSlices.Slice_',num2str(i),'_',num2str(p),';'])
         eval(['oldFieldName = ''Slice_',num2str(i),'_',num2str(p),''';'])
        if sum(loopNests(:,p))==0 || mod(sum(loopNests(:,p)),2)==0 
            newFieldName = [oldFieldName, '_In'];
            [muscSlices.(newFieldName)] =muscSlices.(oldFieldName);
        else
            newFieldName = [oldFieldName, '_Out'];
            [muscSlices.(newFieldName)] =muscSlices.(oldFieldName);   
        end
        muscSlices = rmfield(muscSlices, oldFieldName);
    end
    
end

end
 
%% embedded functions
function [ intersects ] = FindEdgeIntersectsIndex(faces, verts, horizon, horizonCoords, slicePlane)
% FIND INTERSECTS FUNCTION
% NB: prodces intersects = 0 (ERROR STATE) if no intersects found
 % find intersecting poly edges using logic on Z values.
    % first make logical index for each vertex using Z value, 
    % 1 = vertex above slice, 0 = vertex below slices
    intersectIndices = horizonCoords>horizon; 
    % find rows in logical array where all verts in a poly are either 
    % above the slice or below it (p.e non-intersecting polys). This will
    % be cases where the sum of the rows is either zero (all zero) or equal
    % to the number of verts in a poly (all 1)
    [row ~] = find(sum(intersectIndices,2)==0 | ...
        sum(intersectIndices,2)==size(faces,2));
    % ... and then remove these non-intersecting poly verts from the list
    intersectIndices(row,:)=[];
    
    % if no intersecting polys are found (i.e. if this is the triangular
    % part of a compound mesh, and there are no intersections between the
    % current slice horizon and the triangular part of the current mesh),
    % then skip to end and output intersects = 0 (ERROR STATE)
    if isempty(intersectIndices)==1;
        
        intersects = 0;
    
    else
    % use row indices for non-intersecting polys to generate vertical 
    % vector list of actual intersecting poly reference numbers 
    % (row in poly list)
    intersectPolys = (1:size(faces,1))';
    intersectPolys(row,:) = [];
    
    % Use row references to list vert indices from the poly list
    intersectVerts = faces(intersectPolys,:);
    
    % make storage for intersection coordinates (points along the poly
    % edges where the slice plane intersects). Assumes two intersection 
    % points per poly (i.e QUADS AND TRIS ONLY)
    intersects = zeros(size(intersectPolys,1),8);

    for o = 1:size(intersectPolys,1)    
        
        % ASSUMING EITHER QUAD OR TRI MESH! (p.e only two intersecting
        % edges), find the edges (vertex pairs) that are actually
        % intersecting. NB - pairs are (above below) the slice.
        % Poly vert lists go in a loop!
        
        % read current intersect logical index (what verts in current poly
        % are above and what below the slice horizon.
        intersectIndex = intersectIndices(o,:);
        
        % find logical indices for first edge. see notebook for details,
        % basically the logic of the algorithim relies on polygons being a
        % right-handed vertex loop, and the slice horizon passing through
        % two edges (i.e. ONLY QUADS AND TRIS).
        edge1indices = zeros(1,2);
        % This first pair will be consist of the first vertex above the 
        % slice horizon (origin vertex) and the vertex to the LEFT of it in
        % the loop (terminal vertex). i.e looks backwards through the poly 
        % list loop).
        edge1indices(1) = find(intersectIndex==1,1,'first'); 
        % if the origin vertex is not the first vertex in
        % the list, make the vertex preceeding it the terminal vertex (as 
        % we told it to find the first (left to right) entry in the list 
        % equal to 1, anything to the left of it should be equal to zero 
        % unless matlab is having a serious problem).
        if edge1indices(1)>1
            edge1indices(2) = edge1indices(1)-1;
        % if the origin vertix IS the first vertex in the poly list, then 
        % look at the LAST vertex in the list. If it equals zero, make that
        % the terminal vertex...
        elseif intersectIndex(end)==0;
            edge1indices(2) = size(intersectIndex,2);    
        % .. but if the last vertex in the list is 1, NOT zero, then you 
        % have a special case (poly list that looks like this (1 0 0 1) or 
        % this(1 0 1), i.e poly is upside down with respect to the slice
        % horizon, and you need to reverse the search order and take the 
        % vertex to the RIGHT of the origin vertex and the terminal vertex.
       
        elseif  intersectIndex(edge1indices(1)+1)==0;
            edge1indices(2) = edge1indices(1)+1;
        % If this is not zero, then you have something of the form (1101),
        % and upside down quad with a single intercept, and you need to
        % move both parts of edge 1
        else
            edge1indices(1) = edge1indices(1)+1;
            edge1indices(2) = edge1indices(1)+1;
        end
        
        % do similar for second edge pair, but initially looking the RIGHT
        % for the terminal vertex (reversed in the special case described 
        % above).
        edge2indices = zeros(1,2);
        edge2indices(1) = find(intersectIndex==1,1,'last');
        % if origin vertex is not last in list, make the next vertex in the
        % list the terminal.
        if edge2indices(1)<size(intersectIndex,2);
            edge2indices(2) = edge2indices(1)+1;
        % if origin vertex IS last in list, check first vertex in the list
        % is zero, and if so, use that as terminal.
        elseif intersectIndex(1)==0;
            edge2indices(2) = 1;    
        % if the first vertex is also one, special case (1001, 101), so use
        % vertex to the LEFT instead.
        elseif  intersectIndex(edge2indices(1)-1)==0;
            edge2indices(2) = edge2indices(1)-1;
        % If this is not zero, then you have something of the form (1011),
        % and upside down quad with a single intercept, and you need to
        % move both parts of edge 2
        else
            edge2indices(1) = edge2indices(1)-1;
            edge2indices(2) = edge2indices(1)-1;
        end
        
        % use edge indices to retrieve edge vertex coordinates from vertex
        % list.
        edge1Verts = [verts(intersectVerts(o,edge1indices(1)),:) ; ...
            verts(intersectVerts(o,edge1indices(2)),:)]; 
        
        edge2Verts = [verts(intersectVerts(o,edge2indices(1)),:) ; ...
            verts(intersectVerts(o,edge2indices(2)),:)];
        
        % turn vert pair into vector, work out XYZ for intersection of this
        % vector and the slice plane, then output the intersection point!
        edge1Vector = edge1Verts(1,:)-edge1Verts(2,:);
        intersects(o,2:4) = edge1Verts(1,:) - (edge1Vector * ... 
            (edge1Verts(1,slicePlane) - horizon)/ ...
            edge1Vector(slicePlane));
        
        edge2Vector = edge2Verts(1,:)-edge2Verts(2,:);
        intersects(o,6:8) = edge2Verts(1,:) - (edge2Vector * ...
            (edge2Verts(1,slicePlane) - horizon)/ ...
            edge2Vector(slicePlane));
    end
    
    % Check for multiple intersects - these are characterised by intersect
    % pairs in which both intersect points are indentical (one vert of poly
    % is exactly on slice plane, but not other verts or edges interact).
    % Find these and strip them.
    [rows ~] = find(sum((intersects(:,2:4) == ... 
        intersects(:,6:8)),2)==3);
    intersects(rows,:) = [];
    
    % Adds row number as durable ID for each pair of intersection points, 
    % lets them be moved around without losing identification).
    intersects(:,1) = 1:size(intersects,1);
    intersects(:,5) = 1:size(intersects,1);
    
    end
end

function [cn,on] = inpoly(p,node,edge,TOL)

%  INPOLY: Point-in-polygon testing.
%
% Determine whether a series of points lie within the bounds of a polygon
% in the 2D plane. General non-convex, multiply-connected polygonal
% regions can be handled.
%
% SHORT SYNTAX:
%
%   in = inpoly(p,node);
%
%   p   : The points to be tested as an Nx2 array [x1 y1; x2 y2; etc].
%   node: The vertices of the polygon as an Mx2 array [X1 Y1; X2 Y2; etc].
%         The standard syntax assumes that the vertices are specified in
%         consecutive order.
%
%   in  : An Nx1 logical array with IN(i) = TRUE if P(i,:) lies within the
%         region.
%
% LONG SYNTAX:
%
%  [in,on] = inpoly(p,node,edge);
%
%  edge: An Mx2 array of polygon edges, specified as connections between
%        the vertices in NODE: [n1 n2; n3 n4; etc]. The vertices in NODE
%        do not need to be specified in connsecutive order when using the
%        extended syntax.
%
%  on  : An Nx1 logical array with ON(i) = TRUE if P(i,:) lies on a
%        polygon edge. (A tolerance is used to deal with numerical
%        precision, so that points within a distance of
%        eps^0.8*norm(node(:),inf) from a polygon edge are considered "on"
%        the edge.
%
% EXAMPLE:
%
%   polydemo;       % Will run a few examples
%
% See also INPOLYGON

% The algorithm is based on the crossing number test, which counts the
% number of times a line that extends from each point past the right-most
% region of the polygon intersects with a polygon edge. Points with odd
% counts are inside. A simple implementation of this method requires each
% wall intersection be checked for each point, resulting in an O(N*M)
% operation count.
%
% This implementation does better in 2 ways:
%
%   1. The test points are sorted by y-value and a binary search is used to
%      find the first point in the list that has a chance of intersecting
%      with a given wall. The sorted list is also used to determine when we
%      have reached the last point in the list that has a chance of
%      intersection. This means that in general only a small portion of
%      points are checked for each wall, rather than the whole set.
%
%   2. The intersection test is simplified by first checking against the
%      bounding box for a given wall segment. Checking against the bbox is
%      an inexpensive alternative to the full intersection test and allows
%      us to take a number of shortcuts, minimising the number of times the
%      full test needs to be done.
%
%   Darren Engwirda: 2005-2007
%   Email          : d_engwirda@hotmail.com
%   Last updated   : 23/11/2007 with MATLAB 7.0
%
% Problems or suggestions? Email me.

%% ERROR CHECKING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
   TOL = 1.0e-12;
   if nargin<3
      edge = [];
      if nargin<2
         error('Insufficient inputs');
      end
   end
end
nnode = size(node,1);
if isempty(edge)                                                           % Build edge if not passed
   edge = [(1:nnode-1)' (2:nnode)'; nnode 1];
end
if size(p,2)~=2
   error('P must be an Nx2 array.');
end
if size(node,2)~=2
   error('NODE must be an Mx2 array.');
end
if size(edge,2)~=2
   error('EDGE must be an Mx2 array.');
end
if max(edge(:))>nnode || any(edge(:)<1)
   error('Invalid EDGE.');
end

%% PRE-PROCESSING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n  = size(p,1);
nc = size(edge,1);

% Choose the direction with the biggest range as the "y-coordinate" for the
% test. This should ensure that the sorting is done along the best
% direction for long and skinny problems wrt either the x or y axes.
dxy = max(p,[],1)-min(p,[],1);
if dxy(1)>dxy(2)
   % Flip co-ords if x range is bigger
   p = p(:,[2,1]);
   node = node(:,[2,1]);
end
tol = TOL*min(dxy);

% Sort test points by y-value
[y,i] = sort(p(:,2));
x = p(i,1);

%% MAIN LOOP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cn = false(n,1);     % Because we're dealing with mod(cn,2) we don't have
                     % to actually increment the crossing number, we can
                     % just flip a logical at each intersection (faster!)
on = cn;
for k = 1:nc         % Loop through edges

   % Nodes in current edge
   n1 = edge(k,1);
   n2 = edge(k,2);

   % Endpoints - sorted so that [x1,y1] & [x2,y2] has y1<=y2
   %           - also get xmin = min(x1,x2), xmax = max(x1,x2)
   y1 = node(n1,2);
   y2 = node(n2,2);
   if y1<y2
      x1 = node(n1,1);
      x2 = node(n2,1);
   else
      yt = y1;
      y1 = y2;
      y2 = yt;
      x1 = node(n2,1);
      x2 = node(n1,1);
   end
   if x1>x2
      xmin = x2;
      xmax = x1;
   else
      xmin = x1;
      xmax = x2;
   end

   % Binary search to find first point with y<=y1 for current edge
   if y(1)>=y1
      start = 1;
   elseif y(n)<y1
      start = n+1;       
   else
      lower = 1;
      upper = n;
      for j = 1:n
         start = round(0.5*(lower+upper));
         if y(start)<y1
            lower = start;
         elseif y(start-1)<y1
            break;
         else
            upper = start;
         end
      end
   end

   % Loop through points
   for j = start:n
      % Check the bounding-box for the edge before doing the intersection
      % test. Take shortcuts wherever possible!

      Y = y(j);   % Do the array look-up once & make a temp scalar
      if Y<=y2
         X = x(j);   % Do the array look-up once & make a temp scalar
         if X>=xmin
            if X<=xmax

               % Check if we're "on" the edge
               on(j) = on(j) || (abs((y2-Y)*(x1-X)-(y1-Y)*(x2-X))<tol);

               % Do the actual intersection test
               if (Y<y2) && ((y2-y1)*(X-x1)<(Y-y1)*(x2-x1))
                  cn(j) = ~cn(j);
               end

            end
         elseif Y<y2   % Deal with points exactly at vertices
            % Has to cross edge
            cn(j) = ~cn(j);
         end
      else
         % Due to the sorting, no points with >y
         % value need to be checked
         break
      end
   end

end

% Re-index to undo the sorting
cn(i) = cn|on;
on(i) = on;

end      % inpoly()


