function [V,F] = loadWobj(filename)
  %ADAPTED FROM: function [V,F] =
  %read_vertices_and_faces_from_obj_file(filename)
  %BY: Alex Jacobson (http://www.alecjacobson.com/weblog/?p=917)
  % Reads a .obj mesh file and outputs the vertex and face list
  % assumes a 3D triangle mesh and ignores everything but:
  % v x y z and f i j k lines: NB VIV EDITS TO ALSO DEAL WITH QUADS (NOT
  % TOO HARD!)
  % Input:
  %  filename  string of obj file's path
  %
  % Output:
  %  V  number of vertices x 3 array of vertex positions
  %  F  number of faces x 3 array of face indices
  %
  V = zeros(0,3);
  F = zeros(0,4); %ADDED AN EXTRA VALUE HERE (WORKS WITH QUADS)
  vertex_index = 1;
  face_index = 1;
  fid = fopen(filename,'rt');
  line = fgets(fid);
  while ischar(line)
    vertex = sscanf(line,'v %f %f %f');
    face = sscanf(line,'f %d %d %d %d'); % ADDED AN EXTRA VALUE HERE (WORKS WITH QUADS)
    face_long = sscanf(line,'f %d/%d/%d %d/%d/%d %d/%d/%d');

    % see if line is vertex command if so add to vertices
    if(size(vertex)>0)
      V(vertex_index,:) = vertex;
      vertex_index = vertex_index+1;
    % see if line is simple face command if so add to faces
    elseif(size(face)>0)
      F(face_index,1:size(face)) = face; %ADDED DYNAMIC INDEX size(face) TO AVOID DIMENSION MISMATCH
      face_index = face_index+1;
    % see if line is a long face command if so add to faces
    elseif(size(face_long)>0)
      % remove normal and texture indices
      face_long = face_long(1:3:end)
      F(face_index,:) = face_long;
      face_index = face_index+1;
    else
      fprintf('Ignored: %s',line);
    end

    line = fgets(fid);
  end
  fclose(fid);
  
  % IF TRIS NOT QUADS, STRIPS LAST COLUMN (WILL BE ALL ZEROS) FROM FACE
  % LIST, f
  if sum(F(:,4))==0
      F(:,4)=[];
  end
  
end